@@ -2,7 +2,7 @@ from .__about__ import __version__
 
 from .compiler import RDLCompiler
 from .importer import RDLImporter
-from .walker import RDLListener, RDLWalker
+from .walker import RDLListener, RDLWalker, WalkerAction
 from .messages import RDLCompileError
 
 from .node import AddressableNode, VectorNode, SignalNode
