@@ -1,4 +1,4 @@
-from typing import TYPE_CHECKING, List
+from typing import TYPE_CHECKING, List, Set, Tuple
 
 from .perl_preprocessor import PerlPreprocessor
 from .verilog_preprocessor import VerilogPreprocessor
@@ -8,10 +8,11 @@ from .stream import PreprocessedInputStream
 if TYPE_CHECKING:
     from ..compiler import RDLEnvironment
 
-def preprocess_file(env: 'RDLEnvironment', path: str, search_paths: List[str]) -> PreprocessedInputStream:
+def preprocess_file(env: 'RDLEnvironment', path: str, search_paths: List[str]) -> Tuple[PreprocessedInputStream, Set[str]]:
     # Run file through Perl preprocessor
     ppp = PerlPreprocessor(env, path, search_paths)
     preprocessed_text, seg_map = ppp.preprocess()
+    included_files = ppp.included_files
 
     # ... then through the Verilog preprocessor
     vpp = VerilogPreprocessor(env, preprocessed_text, seg_map)
@@ -21,4 +22,4 @@ def preprocess_file(env: 'RDLEnvironment', path: str, search_paths: List[str]) -
 
     # Encapsulate into an Antlr-like input stream object
     input_stream = PreprocessedInputStream(preprocessed_text, seg_map)
-    return input_stream
+    return input_stream, included_files
